@@ -1,11 +1,11 @@
 // 2003-02-26 Benjamin Kosnik <bkoz@redhat.com>
 
-// Copyright (C) 2003 Free Software Foundation, Inc.
+// Copyright (C) 2003, 2009 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -14,9 +14,8 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
 // IA 64 C++ ABI - 5.1 External Names (a.k.a. Mangling)
 
@@ -27,9 +26,8 @@ int main()
 {
   using namespace __gnu_test;
 
- // cplus-dem FAIL
-  verify_demangle("_Z3fooA30_A_i", "foo(int [30][])");
-
+  // cplus-dem FAIL
+  verify_demangle("_Z3kooPA28_A30_i", "koo(int (*) [28][30])");
 
   return 0;
 }
